#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import sys, os, re

from Argumental.Argue import Argue
from Perdy.parser import printXML
from GoldenChild.xpath import *
from Swapsies.xmi import XMI

args = Argue()

#_________________________________________________________________
@args.command(single=True)
class STEP2UML(object):
	'''
	convert STEP to UML
	'''

	def __init__(self):
		self.base_types = dict() # validation: xmi.class
		self.lovs = dict() # id: xmi.class
		self.attributes = dict() # id: xml.class
		self.user_types = dict() # id: xmi.class
		self.references = dict() # id: xmi.class

	def References(self, user_type):
		pass
	
	def Attributes(self, user_type):
		pass
	
	def UserTypes(self, user_type):
		pass
	
	@args.operation
	@args.parameter(name='file', help='input step.xml file')
	@args.parameter(name='output', short='o', help='output xmi UML file')
	def toUML(self, file, output=None):
		'''
		make an UML XMI file from a STEP.XML input
		'''
		print(file)

		spi = 'STEP-ProductInformation'

		STEP = XML(*getContextFromFile(file))

		xmi = XMI()

		# make the fundamentals
		classes = xmi.makePackage('Fundamentals', xmi.modelNS)
		diagram = xmi.makeClassDiagram('Fundamentals', classes)

		for base_type in ['text','number','date','datetime','isodate','isodatetime']:
			_base_type = xmi.makeClass(base_type, classes, uid=base_type)
			xmi.addDiagramClass(_base_type, diagram)
			self.base_types[base_type] = _base_type

		# get LOVs as class enums
		classes = xmi.makePackage('ListsOfValues', xmi.modelNS)
		diagram = xmi.makeClassDiagram('ListsOfValues', classes)

		for lov in getElements(STEP.ctx, f'/{spi}/ListsOfValues/ListOfValue'):
			lname = getElementText(STEP.ctx, 'Name', lov)
			lid = getAttribute(lov, 'ID')
			_lov = xmi.makeClass(lname, classes, uid=lid)
			xmi.addDiagramClass(_lov, diagram)
			#print(lid, lname)
			self.lovs[lid] = _lov
			
		# get attributes as classes
		classes = xmi.makePackage('Attributes', xmi.modelNS)
		diagram = xmi.makeClassDiagram('Attributes', classes)

		for attribute in getElements(STEP.ctx, f'/{spi}/AttributeList/Attribute'):
			aname = getElementText(STEP.ctx, 'Name', attribute)
			aid = getAttribute(attribute, 'ID')
			_attribute = xmi.makeClass(aname, classes, uid=aid)
			xmi.addDiagramClass(_attribute, diagram)
			#print(aid, aname)
			self.attributes[aid] = _attribute
					
		# make the user types
		classes = xmi.makePackage('UserTypes', xmi.modelNS)
		diagram = xmi.makeClassDiagram('UserTypes', classes)
		
		for user_type in getElements(STEP.ctx, f'/{spi}/UserTypes/UserType'):
			uname = getElementText(STEP.ctx, 'Name', user_type)
			uid = getAttribute(user_type, 'ID')
			_user_type = xmi.makeClass(uname, classes, uid=uid)
			xmi.addDiagramClass(_user_type, diagram)
			#print(uid, uname)
			self.user_types[uid] = _user_type

			xpath=f'/{spi}/AttributeList/Attribute[UserTypeLink/@UserTypeID="{uid}"]'
			for attribute in getElements(STEP.ctx, xpath):
				aname = getElementText(STEP.ctx, 'Name', attribute)
				aid = getAttribute(attribute, 'ID')
				multiple = getAttribute(attribute, 'MultiValued') == 'true'				
				tipe = getElement(STEP.ctx, 'Validation', attribute)
				if tipe:
					cid = getAttribute(tipe, 'BaseType')
					ctype = self.base_types[cid]
				else:
					tipe = getElement(STEP.ctx, 'ListOfValueLink', attribute)
					if tipe:
						cid = getAttribute(tipe, 'ListOfValueID')
						ctype = self.lovs[cid]
				xmi.makeAttribute(aname, ctype, None, _user_type, array=multiple)

			self.user_types[uid] = _user_type

		# make references
		classes = xmi.makePackage('References', xmi.modelNS)
		diagram = xmi.makeClassDiagram('References', classes)

		for reference in getElements(STEP.ctx, f'/{spi}/CrossReferenceTypes/ProductCrossReferenceType'):
			rname = getElementText(STEP.ctx, 'Name', reference)
			rid = getAttribute(reference, 'ID')
			_reference = xmi.makeClass(rname, classes, uid=rid)
			xmi.addDiagramClass(_reference, diagram)
			#print(rid, rname)

			for user_type_link in getElements(STEP.ctx, 'UserTypeLink', reference):
				uid = getAttribute(user_type_link, 'UserTypeID')
				_user_type = self.user_types[uid]
				xmi.makeAssociation(rname, _user_type, _reference, classes)

			for target_user_type_link in getElements(STEP.ctx, 'TargetUserTypeLink', reference):
				tid = getAttribute(target_user_type_link, 'UserTypeID')
				_target_user_type = self.user_types[tid]
				xmi.makeAssociation(rname, _reference, _target_user_type, classes)
				
			self.references[rid] = _reference


		'''									   
		parameters = {
			'name' : string,
			'child' : child
		}
		returns=child
		xmi.makeOperation('getChild',parameters,returns,parent)

		xmi.makeStereotype('hifi',string)

 		diagram = xmi.makeClassDiagram('Diagram',classes)
		xmi.addDiagramClass(parent,diagram)
		xmi.addDiagramClass(child,diagram)
		'''
											   
		# export the results

		name = output or f'{file}.xmi'
		with open(name,'w') as _output:
			printXML(str(xmi.doc), output=_output, colour=False)

#_________________________________________________________________
if __name__ == '__main__': args.execute()




#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import sys, re, os, json, logging, codecs, ssl

from datetime import datetime
from collections import OrderedDict
from suds.client import Client
from suds.sax.text import Text
from suds.wsse import Security, UsernameToken
from suds.xsd.sxbasic import Complex
from dotmap import DotMap

from Argumental.Argue import Argue
from Spanners.Squirrel import Squirrel
from Perdy.pretty import prettyPrint
from Baubles.Logger import Logger

args = Argue()
squirrel = Squirrel()
logger = Logger()

def quietly():
	for name in [
		'boto3.resources.action',
		'boto3.resources.factory',
		'botocore.auth',
		'botocore.client',
		'botocore.credentials',
		'botocore.endpoint',
		'botocore.hooks',
		'botocore.loaders',
		'botocore.parsers',
		'botocore.retryhandler',
		'pyxb.binding.basis',
		'suds.client',
		'suds.metrics',
		'suds.mx.core',
		'suds.mx.literal',
		'suds.resolver',
		'suds.transport.http',
		'suds.umx.typed',
		'suds.wsdl',
		'suds.xsd.deplist',
		'suds.xsd.query',
		'suds.xsd.schema',
		'suds.xsd.sxbase',
		'suds.xsd.sxbasic',
	]:
		logging.getLogger(name).setLevel(logging.CRITICAL)
	

quietly()

@args.argument(short='o', help='output to file')
def output(): return

@args.argument(short='c', flag=True, help='in colour')
def colour(): return False

#_________________________________________________________________
@args.command(single=True)
class StepSoapClient(object):

	@args.property(short='H', default='http://host')
	def hostname(self): return
	
	@args.property(short='w', default='StepWS/stepws?wsdl')
	def wsdlpath(self): return

	@args.property(short='u', default='stepsys')
	def username(self): return

	@args.property(short='p')
	def password(self):
		return squirrel.get('%s:%s'%(self.hostname, self.username))
	

	@args.property(short='v', flag=True)
	def verbose(self): return False

	#.............................................................
	def __init__(self, hostname=None, wsdlpath=None, username=None, password=None, verbose=False):
		if hostname: self.hostname = hostname
		if wsdlpath: self.wsdlpath = wsdlpath
		if username: self.username = username
		if password: self.password = password
		if verbose:  self.verbose  = verbose
		
		if self.verbose:
			logger.setLevel(logging.DEBUG)
		else:
			logger.setLevel(logging.INFO)

		if hasattr(ssl, '_create_unverified_context'):
			ssl._create_default_https_context = ssl._create_unverified_context
			
		self.client = Client('%s/%s'%(self.hostname, self.wsdlpath), cache=None)
		#logger.debug('client=%s',self.client)
		
		self._types = dict(map(lambda x: 
			(x[0].name, x[0]), self.client.sd[0].types
		))
		
		return


	#.............................................................
	@args.operation
	def service(self):
		print(self.client)


	#.............................................................
	def createAccessControl(self):
		request = self.client.factory.create('ns1:accessContext')
		request.userName = self.username
		request.password = self.password
		return request
	

	#.............................................................
	@args.operation
	def directory(self):
		'''
		print out the client specification
		'''
		return sorted(self.client.sd[0].service.ports[0].methods.keys())
	

	#.............................................................
	@args.operation
	def types(self):
		'''
		print out the client types
		'''
		return sorted(self._types.keys())
		

	#.............................................................
	def typo(self, part):
		name = str(part[0].name)
		if name not in self._types.keys():
			name = '%sType'%name
		if name not in self._types.keys():
			return name
		t = self._types[name]
		d = dict()
		if isinstance(t, Complex):
			d[t.name] = map(self.typo, t.children())
		return d
		

	#.............................................................
	@args.operation
	def describe(self, name):
		'''
		describe a method
		'''
		method = self.client.sd[0].service.ports[0].methods[name]
		result = {
			method.name: dict(
				input=self.typo(
					method.soap.input.body.parts
				),
				output=self.typo(
					method.soap.output.body.parts
				),
				faults=self.typo(
					method.soap.faults[0].parts
				)
			)
		}
		#print method.soap
		return(result)
			

	#.............................................................
	@args.operation
	def dummy(self, name):
		'''
		describe a method
		'''
		request = self.createAccessControl()
		return self.client.service.dummy(request,name)


	#.............................................................
	@args.operation
	def getWorkspaces(self):
		response = self.client.service.getWorkspaces(self.username,self.password)
		return response


	#.............................................................
	@args.operation
	def approve(self, nodeURL):
		request = self.createAccessControl()
		response = self.client.service.approve(request, nodeURL)
		return response


	#.............................................................
	@args.operation
	def getBaseProduct(self, nodeURL):
		request = self.createAccessControl()
		response = self.client.service.getBaseProduct(request, nodeURL)
		return response
		

	#.............................................................
	@args.operation
	def createProduct(self, productID, name, objectTypeURL, parentURL, beforeURL=None):
		request = self.createAccessControl()
		response = self.client.service.createProduct(request,productID, name, objectTypeURL, parentURL, beforeURL) 
		return response
													 
	#.............................................................
	@args.operation
	def createClassification(self, classificationID, name, objectTypeURL, parentURL, beforeURL=None):
		request = self.createAccessControl()
		response = self.client.service.createClassification(request, classificationID, name, objectTypeURL, parentURL, beforeURL) 
		return response


	#.............................................................
	@args.operation
	def getClassifications(self, nodeURL):
		request = self.createAccessControl()
		return self.client.service.getClassifications(request, nodeURL)
		

	#.............................................................
	@args.operation
	def addClassification(self, nodeURL, classificationURL):
		request = self.createAccessControl()
		return self.client.service.addClassification(request, nodeURL, classificationURL)
		

	#.............................................................
	@args.operation
	def getAttributeDetails(self, nodeURL):
		request = self.createAccessControl()
		return self.client.service.getAttributeDetails(request, nodeURL)
		
	#.............................................................
	@args.operation
	@args.parameter(name='urls', short='u', flag=True, help='show attribute urls')
	@args.parameter(name='useNames', short='n', flag=True, help='use attribute name not id')
	@args.parameter(name='ignoreNulls', short='i', flag=True)
	def getValues(self, nodeURL, urls=False, useNames=False, ignoreNulls=False):
		request = self.createAccessControl()
		response = self.client.service.getValues(request, nodeURL)
		values = dict()
		for value in response:
			v = getattr(value,'value',None)
			if not v and ignoreNulls:
				continue
			if v:
				v='%s'%v

			if urls:
				values[value['attributeURL']] = v
			elif useNames:
				values[value['attributeTtitle']] = v
			else:
				values[value['attributeURL'].split('=')[1]] = v
		return values


	#.............................................................
	@args.operation
	@args.parameter(name='nameEqValues', short='e', nargs='*', metavar='name=value', help='name is the attribute id')
	def setValues(self, nodeURL, nameEqValues=[]):
		request = self.createAccessControl()
		ns1_values = list()
		for nameEqValue in nameEqValues:
			parts = nameEqValue.split('=')
			[name,value] = [parts[0],'='.join(parts[1:])]
			value = value.replace('<','&lt;').replace('>','&gt;')
			value = value.replace('&','&amp;')
			#value = '<![CDATA[%s]]>'%value
			ns1_value = self.client.factory.create('ns1:value')
			ns1_value.attributeURL = 'step://attribute?id=%s'%name
			ns1_value.value = value
			ns1_values.append(ns1_value)

			#print(dir(ns1_value))
		response = self.client.service.setValues(request, nodeURL, ns1_values)
		return response


	#.............................................................
	@args.operation
	def getName(self, nodeURL):
		request = self.createAccessControl()
		response = self.client.service.getName(request, nodeURL)
		return response


	#.............................................................
	@args.operation
	@args.parameter(name='caseSensitive', short='i', flag=True)
	@args.parameter(name='sort', short='s', flag=True)
	@args.parameter(name='offset', type=int, default=0)
	@args.parameter(name='maxCount', type=int, default=100)
	def queryById(self, queryString, caseSensitive=False, sort=False, offset=0, maxCount=100):
		request = self.createAccessControl()
		response = self.client.service.queryById(request, queryString, caseSensitive, sort, offset, maxCount)
		return response


	#.............................................................
	@args.operation
	@args.parameter(name='caseSensitive', flag=True)
	@args.parameter(name='offset', type=int, default=0)
	@args.parameter(name='maxCount', type=int, default=100)
	def queryByAttribute(self, attributeID, attributeValue, caseSensitive=False, offset=0, maxCount=100):
		request = self.createAccessControl()
		response = self.client.service.queryByAttribute(request, 'step://attribute?id=%s'%attributeID, attributeValue, caseSensitive, offset, maxCount)
		return response


	#.............................................................
	@args.operation
	def getNodeDetails(self, nodeURL):
		request = self.createAccessControl()
		try:
			response = self.client.service.getNodeDetails(request, nodeURL)
			return response
		except:
			logger.debug(sys.exc_info()[1])
		return
		

	#.............................................................
	@args.operation
	def moveNode(self, nodeURL, oldParentURL, newParentURL):
		request = self.createAccessControl()
		response = self.client.service.moveNode(request, nodeURL, oldParentURL, newParentURL)
		return response


	#.............................................................
	@args.operation
	def setName(self, nodeURL, name=None):
		request = self.createAccessControl()
		response = self.client.service.setName(request, nodeURL, name)
		return response


	#.............................................................
	@args.operation
	def getChildren(self, nodeURL):
		request = self.createAccessControl()
		response = self.client.service.getChildren(request, nodeURL)
		return response
		

	#.............................................................
	@args.operation
	def getValidChildTypes(self, nodeURL):
		request = self.createAccessControl()
		response = self.client.service.getValidChildTypes(request, nodeURL)
		return response
		

	#.............................................................
	@args.operation
	def getPath(self, nodeURL):
		request = self.createAccessControl()
		response = self.client.service.getPath(request, nodeURL)
		return response


	#.............................................................
	@args.operation
	def getReferenceTypes(self):
		request = self.createAccessControl()
		response = self.client.service.getReferenceTypes(request)
		return response


	#.............................................................
	@args.operation
	def createReference(self, ownerURL, targetURL, referenceType):
		request = self.createAccessControl()
		response = self.client.service.createReference(request, ownerURL, targetURL, referenceType)
		return response


	#.............................................................
	@args.operation
	def getReferences(self, nodeURL):
		request = self.createAccessControl()
		response = self.client.service.getReferences(request, nodeURL)
		return response


	#.............................................................
	@args.operation
	def deleteNode(self, nodeURL):
		request = self.createAccessControl()
		response = self.client.service.deleteNode(request, nodeURL)
		return response
	

	#.............................................................
	@args.operation
	def getTasks(self, nodeURL, workflowID):
		request = self.createAccessControl()
		response = self.client.service.getTasks(request, nodeURL, workflowID)
		return response


	#.............................................................
	@args.operation
	def getWorkflowProcesses(self, workflowID):
		request = self.createAccessControl()
		response = self.client.service.getWorkflowProcesses(request, workflowID)
		return response


	#.............................................................
	@args.operation
	def getWorkflowProcessDetail(self, processTemplateIdType, processIdType):
		request = self.createAccessControl()
		response = self.client.service.getWorkflowProcessDetail(request, processTemplateIdType, processIdType)
		return response


	#.............................................................
	@args.operation
	def startWorkflow(self, nodeID, workflowID, message):
		request = self.createAccessControl()
		response = self.client.service.startWorkflow(request, nodeID, workflowID, message)
		return response


	#.............................................................
	@args.operation
	def getLOVValueIDs(self,nodeID):
		request = self.createAccessControl()
		response = self.client.service.getLOVValueIDs(request,nodeID)
		return response


#_________________________________________________________________
def render(node):
	#sys.stderr.write('%s[%s]\n'%(node,type(node)))
	if type(node) in [Text]:
		return '%s'%node
	if type(node) == list: 
		return list(map(render, node))
	if hasattr(node, '__dict__'):
		node = Client.dict(node)
	if type(node) in [dict, OrderedDict]:
		d = dict()
		for key in node.keys():
			d[key] = render(node[key])
		return d
	return node




#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import sys, json

from STEP.REST import *

result = args.execute()
if type(result) in [list, dict]:
	json.dump(result, sys.stdout, indent=4)
else:
	print(result)




#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import sys, os, argparse, json, xmltodict, codecs, sqlalchemy

from datetime import datetime, date
from decimal import Decimal
from io import StringIO
from collections import namedtuple, OrderedDict
from xlrd import open_workbook, xldate_as_tuple

from GoldenChild.xpath import *
from Perdy.pyxbext import directory
from Perdy.parser import printXML
from Spanners.IdentityCache import IdentityCache
from Spanners.Squirrel import Squirrel
from Argumental.Argue import Argue

from STEP.XSD import *

args = Argue()

#_________________________________________________________________
@args.command(single=True)
class Converter(object):
	'''
	tool to convert excel into step xml formats

	'''
	
	@args.property(default='PIM', help='prefix to use on ID definitions')
	def prefix(self): return

	@args.property(default='Context1', help='step context id')
	def context(self): return

	@args.property(default='Main', help='step workspace id')
	def workspace(self): return

	parent_type_id = 'Product user-type root'
	parent_product_id = 'Product hierarchy root' 

	#_____________________________________________________________
	def __init__(self):
		self.ids = IdentityCache()
		
		self.types   = OrderedDict()
		self.ag      = None    # parent attribute group
		self.ags     = OrderedDict()
		self.lovs	 = OrderedDict()
		self.attrs   = OrderedDict()
		self.prods   = OrderedDict()

		self.doc = STEP_ProductInformation(
			ExportTime = datetime.now(),
			ContextID = self.context,
			WorkspaceID = self.workspace,
			UseContextLocale=False,
			ListOfValuesGroupList = ListOfValuesGroupListType(),
			ListsOfValues = ListsOfValuesType(),
			AttributeGroupList = AttributeGroupListType(),
			AttributeList = AttributeListType(),
			UserTypes = UserTypesType(),
			CrossReferenceTypes = CrossReferenceTypesType(),
			Products = ProductsType(),
			Classifications = ClassificationsType(),
		)

		
	#_____________________________________________________________
	def __del__(self):
		self.close()


	#_____________________________________________________________
	def close(self):
		if self.ids:
			self.ids.save()
		

	#_____________________________________________________________
	def save(self, file):
		with codecs.open(file, 'w', encoding='utf8') as output:
			printXML(self.doc.toxml(), output=output)
					

	#_____________________________________________________________
	def make_prod(self, id, name, type_id, parent_id):
		
		self.prods[id] = ProductType(
			ID = id,
			Name = [NameType(name)],
			UserTypeID = type_id,
			ParentID = parent_id,
		)
		self.doc.Products.append(self.prods[id])

		return self.prods[id]

				
	#_____________________________________________________________
	def make_type(self, id, name, parent_type_id):

		self.types[id] = UserTypeType(
			ID = id,
			Name = [NameType(name)],
			AllowInDesignTemplate='false',
			AllowQuarkTemplate='false',
			IDPattern='%s_[uuid]'%self.prefix,
			IsCategory='true',
			ManuallySorted='false',
			ReferenceTargetLockPolicy='Strict',
			Referenced='true',
			UserTypeLink = [
				UserTypeLinkType(
					UserTypeID=parent_type_id
				)
			]
		)
		self.doc.UserTypes.append(self.types[id])

		self.ags[id] = AttributeGroupType(
			ID = id,
			ShowInWorkbench = 'true',
			ManuallySorted = 'false',
			Name = [
				NameType(name)
			]
		)
		self.ag.AttributeGroup.append(self.ags[id])

		return self.types[id]


	#_____________________________________________________________
	def make_attr(self, id, name, attr_type, length, parent_group_id, user_type_id):

		if id not in self.attrs.keys():
			attr = AttributeType(
				ID = id, 
				Name = [NameType(name)],
				MultiValued = 'false',
				ProductMode = 'Normal', #specification, use "Property" for description
				FullTextIndexed = 'false',
				ExternallyMaintained = 'false',
				Derived = 'false',

				AttributeGroupLink = [
					AttributeGroupLinkType(
						AttributeGroupID = parent_group_id
					)
				],

				MetaData = MetaDataType(
				),
				
				UserTypeLink = [
					UserTypeLinkType(
						UserTypeID = user_type_id
					)
				]
			)

			lov = None
			if lov:
				attr.ListOfValueLink = ListOfValueLinkType(
					ListOfValueID=lov
				)
			else:
				attr.Validation = ValidationType(
					BaseType = attr_type,
					MaxLength = length,
				)

			tooltip = None
			if tooltip:
				attr.MetaData.append(ValueType(
					tooltip,
					AttributeID='ToolTip'
				))
								 
			self.doc.AttributeList.append(attr)
			self.attrs[id] = attr

		return self.attrs[id]
	
		
	#_____________________________________________________________
	def make_ref(self, id, name, source, target):
			
		reference = ProductCrossReferenceTypeType(
			ID = id,
			Name = [NameType(name)],
			UserTypeLink = [
				UserTypeLinkType(
					UserTypeID = source
				)
			],
			TargetUserTypeLink = [
				TargetUserTypeLinkType(
					UserTypeID = target
				)
			]
		)
		
		self.doc.CrossReferenceTypes.append(reference)

			
	#_____________________________________________________________
	def make_lov(self, id, name, tipe, length, parent):
		if id not in self.lovs.keys():
			lov = ListOfValueType(
				ID = id,
				Name = [NameType(name)],
				ParentID = parent,
				UseValueID = 'true',
				AllowUserValueAddition = 'false',
				Validation = ValidationType(
					BaseType=tipe, 
					MaxLength=length
				),
				Value = [
				]
			)	
			self.doc.ListsOfValues.append(lov)
			self.lovs[id] = lov
			
		return self.lovs[id]


	#_____________________________________________________________
	@args.operation
	@args.parameter(name='id', help='LOV ID')
	@args.parameter(name='name', help='LOV Name')
	@args.parameter(name='input', help='excel file with attribute definitions')
	@args.parameter(name='validation', short='V', default='TEXT', help='attribute validation type')
	@args.parameter(name='length', short='L', type=int, default=256, help='attribute validation length')
	@args.parameter(name='output', short='o', default='step.xml', help='step xml format output file')
	def make_lovs(self, id, name, input, validation=None, length=None, output=None):
		'''
		process an excel file to create a STEP LOV xml import

		| ID | Name |
		|----+------|
		| a  | aye  |
		| b  | bee  |
		|----+------|
 
		<group_id=sheet_name/>

		'''

		workbook = open_workbook(filename=input)

		for sheet in workbook.sheets():

			lov_group = ListOfValuesGroupType(
				ID=sheet.name,
				Name=[NameType(sheet.name)],
				ParentID='List Of Values group root',
			)
			self.doc.ListOfValuesGroupList.append(lov_group)

			lov = self.make_lov(id, name, validation, length, lov_group.ID)

			for r in list(range(sheet.nrows))[1:]:
				if sheet.ncols > 1:
					value_id = sheet.cell(r,0).value
					value_name = sheet.cell(r,1).value
				else:
					value_name = sheet.cell(r,0).value

				lov.Value.append(ValueType(value_name, ID=value_id))
		
		self.save(output)
				
	#_____________________________________________________________
	@args.operation
	@args.parameter(name='input', help='excel file with attribute definitions')
	@args.parameter(name='output', short='o', default='step.xml', help='step xml format output file')
	def process(self, input, output=None):
		'''
		process an excel file to create a STEP import

		| Class  | CSV        | Name | Type    | Length | 
		|--------+------------+------+---------+--------|
		| Group1 | group1.csv | id   | varchar | 40     |
		| Group1 | group1.csv | name | varchar | 256    |

		'''
		
		self.ag = AttributeGroupType(
			ID = self.prefix,
			ShowInWorkbench = 'true',
			ManuallySorted = 'false',
			Name = [
				NameType(self.prefix)
			]
		)
		self.doc.AttributeGroupList.append(self.ag)

		root_id = '%s_Root'%self.prefix
		root_name = '%s Root'%self.prefix

		self.tipe = self.make_type(root_id, root_name, self.parent_type_id)
		self.prod = self.make_prod(root_id, root_name, self.tipe.ID, self.parent_product_id)

		columns = {
			'' : 'text',
			'bigint' : 'integer',
			'date' : 'isodate',
			'datetime' : 'isodatetime',
			'flag': 'text',
			'float' : 'fraction',
			'image url': 'text',
			'int' : 'integer',
			'integer' : 'integer',
			'long' : 'integer',
			'lookup table': 'text',
			'money' : 'number',
			'ntext'  : 'text',
			'number' : 'number',
			'numeric' : 'number',
			'nvarchar' : 'text',
			'relationship' : 'text',
			'string enumeration' : 'text',
			'string' : 'text',
			'thumbnail image url': 'text',
			'unicode' : 'text',
			'varchar' : 'text',
			'varchar2' : 'text',
		}

		workbook = open_workbook(filename=input)

		sheet = workbook.sheets()[0]

		Dictionary = namedtuple(
			'Dictionary',
			list(map(
				lambda x: sheet.cell(0, x).value,
				list(range(sheet.ncols))
			))
		)

		Source = None
		CSV = None
		
		for r in list(range(sheet.nrows))[1:]:
			cols = list(map(
				lambda x: sheet.cell(r, x).value,
				list(range(sheet.ncols))
			))
			row = Dictionary(*cols)

			if row.Source != Source:
				Source = row.Source
				print('%s'%Source)
				
				root_path = 'Root_%s'%(row.Source)
				root_id  = '%s_%s'%(self.prefix, self.ids.get(root_path))
				root_name = root_path.replace('_',' ')
				root     = self.make_type(root_id, root_name, self.tipe.ID)
				roots     = self.make_prod(root.ID, '%s'%Source, root.ID, self.prod.ID)

			if row.CSV != CSV:
				CSV = row.CSV
					  
				types_path = 'Node_%s_%s'%(row.Source, row.CSV)
				types_id   = '%s_%s'%(self.prefix, self.ids.get(types_path))
				types_name = types_path.replace('_',' ')
				tipes      = self.make_type(types_id, types_name, root.ID)
				print('\t%s'%types_path)

				type_path = 'Leaf_%s_%s'%(row.Source, row.CSV)
				type_id   = '%s_%s'%(self.prefix, self.ids.get(type_path))
				type_name = type_path.replace('_',' ')
				tipe      = self.make_type(type_id, type_name, tipes.ID)
				print('\t\t%s'%type_path)

				prods = self.make_prod(tipes.ID, '%s*'%row.CSV, tipes.ID, roots.ID)
				prod  = self.make_prod(tipe.ID, '%s'%row.CSV, tipe.ID, prods.ID)

			attr_path = '%s_%s_%s'%(row.Source, row.CSV, row.Name)
			attr_id   = '%s_%s'%(self.prefix, self.ids.get(attr_path))
			attr_name = row.Name
			attr_type = columns[row.Type.lower()]
			length = str(row.Length).split('.')[0]
			if len(length) > 0:
				attr_len  = length
			else:
				attr_len = None
				
			print('\t\t\t%s'%attr_path)

			attr = self.make_attr(attr_id, attr_name, attr_type, attr_len, self.ags[tipe.ID].ID, tipe.ID)
			prod.AttributeLink.append(
				AttributeLinkType(
					AttributeID = attr.ID
				)
			)

		for attribute in self.attrs.values():
			self.prod.AttributeLink.append(
				AttributeLinkType(
					AttributeID=attribute.ID
				)
			)
			
		self.save(output)





#!/usr/bin/env python3

from collections import OrderedDict
from suds.sudsobject import asdict, Factory as SudsFactory

def suds2dict(d):
    """
    Suds object serializer
    Borrowed from https://stackoverflow.com/questions/2412486/serializing-a-suds-object-in-python/15678861#15678861
    """
    if type(d) in [dict, OrderedDict]: return d

    out = {'__class__': d.__class__.__name__}
    for k, v in asdict(d).items():
        if hasattr(v, '__keylist__'):
            out[k] = suds2dict(v)
        elif isinstance(v, list):
            out[k] = []
            for item in list(v):
                if hasattr(item, '__keylist__'):
                    out[k].append(suds2dict(item))
                else:
                    out[k].append(item)
        else:
            out[k] = v
    return out


def dict2suds(d):
    """
    Suds object deserializer
    """
    out = {}
    for k, v in d.items():
        if isinstance(v, dict):
            out[k] = dict2suds(v)
        elif isinstance(v, list):
            out[k] = []
            for item in list(v):
                if isinstance(item, dict):
                    out[k].append(dict2suds(item))
                else:
                    out[k].append(item)
        else:
            out[k] = v
    return SudsFactory.object(out.pop('__class__'), out)




#!/usr/bin/env python3

# PYTHON_ARGCOMPLETE_OK

import os, sys, re, json, requests, xmltodict, shutil

from copy import copy
from datetime import datetime
from io import StringIO, BytesIO
from dotmap import DotMap

from Perdy.parser import doParse, printXML
from Perdy.pretty import prettyPrint
from Argumental.Argue import Argue
from Spanners.Squirrel import Squirrel

args = Argue()
squirrel = Squirrel()

dts = '%Y-%m-%dT%H:%M:%S'


#________________________________________________________________
class Silencer(object):
	def write(self, *args, **kwargs):
		pass
	def close(self):
		pass
	def flush(self):
		pass


#________________________________________________________________
@args.command(name='step')
class STEP(object):
	'''
	base class to store the common properties and operations
	'''
	
	@args.property(short='H', default='http://host')
	def hostname(self): return

	@args.property(short='U', default='stepsys')
	def username(self): return

	@args.property(short='P')
	def password(self): return squirrel.get('stibo:%s:%s'%(self.hostname, self.username))

	@args.property(short='v', flag=True)
	def verbose(self): return

	@args.property(short='x', flag=True, help='output in xml')
	def asXML(self): return

	@args.property(short='o', help='output to a file')
	def output(self): return None

	@args.property(short='C', default='Context1')
	def context(self): return
	
	@args.property(short='W', default='Main')
	def workspace(self): return

	@args.property(short='V')
	def version(self): return

	@args.property(short='F')
	def format(self): return

	@args.property(short='X')
	def xslt(self): return

	def __init__(self, asXML=None, verbose=None, output=None, silent=True):
		if asXML: self.asXML = asXML
		if verbose: self.verbose = verbose
		if output: self.output = output
		self.silent = silent
		tipe = 'application/xml' if self.asXML else 'application/json'
		self.headers={
			'Accept': tipe,
			#'Content-Type': tipe
		}
		self.path = 'restapi' if self.asXML else 'restapiv2'

		
	def process(self, path, params=None, kwargs=dict()):
		'''
		shared get operation
		'''
		url = '%s/%s/%s'%(self.hostname, self.path, path)
		auth= (self.username, self.password)
		params={
			"context" : self.context,
			"workspace" : self.workspace
		}
		if self.verbose:
			json.dump(dict(url=url, headers=self.headers, params=params), sys.stderr, indent=4)
		result = requests.get(url=url, auth=auth, headers=self.headers, params=params, **kwargs)
		if result.status_code != 200 or self.verbose:
			sys.stderr.write('%s: %s\n'%(result, result.text))
		return result

	
	def get(self, path, params=None):
		result = self.process(path, params, kwargs=dict())
		return self.export(result.text)

	
	def put(self, path, body=None, params=None, headers=None):
		url = '%s/%s/%s'%(self.hostname, self.path, path)
		auth= (self.username, self.password)
		if not params:
			params={
				"context" : self.context,
				"workspace" : self.workspace
			}
		if not headers:
			headers = copy(self.headers)
			headers['Content-Type'] = headers['Accept']
		if self.verbose:
			json.dump(dict(url=url, headers=headers, params=params), sys.stderr, indent=4)
		result = requests.put(url=url, auth=auth, headers=headers, params=params, data=body)
		if result.status_code != 200 or self.verbose:
			sys.stderr.write('%s: %s\n'%(result, result.text))
		return result

	
	def post(self, path, body=None, params=None, headers=None):
		url = '%s/%s/%s'%(self.hostname, self.path, path)
		auth= (self.username, self.password)
		if not params:
			params={
				"context" : self.context,
				"workspace" : self.workspace
			}
		if not headers:
			headers = copy(self.headers)
			headers['Content-Type'] = headers['Accept']
		if self.verbose:
			json.dump(dict(url=url, headers=headers, params=params), sys.stderr, indent=4)
		result = requests.post(url=url, auth=auth, headers=headers, params=params, data=body)
		if result.status_code not in [200,201] or self.verbose:
			sys.stderr.write('%s: %s\n'%(result, result.text))
		return result


	def delete(self, path, body=None, params=None, headers=None):
		url = '%s/%s/%s'%(self.hostname, self.path, path)
		auth= (self.username, self.password)
		if not params:
			params={
				"context" : self.context,
				"workspace" : self.workspace
			}
		if not headers:
			headers = copy(self.headers)
			headers['Content-Type'] = headers['Accept']
		if self.verbose:
			json.dump(dict(url=url, headers=headers, params=params), sys.stderr, indent=4)
		result = requests.delete(url=url, auth=auth, headers=headers, params=params, data=body)
		if result.status_code != 200 or self.verbose:
			sys.stderr.write('%s: %s\n'%(result, result.text))
		return result

	
	def file(self, path, destination):
		params={
			"context" : self.context,
			"workspace" : self.workspace
		}
		self.headers = None
		result = self.process(path, params=params, kwargs=dict(stream=True))
		with open(destination, 'wb') as output:
			result.raw.decode_content = True
			shutil.copyfileobj(result.raw, output)
		return


	def export(self, result):
		if not result: return
		result = result if self.asXML else json.loads(result)

		if self.output:
			sys.stderr.write(self.output)
			dir = os.path.dirname(self.output)
			if dir and len(dir) > 0 and not os.path.isdir(dir):
				os.makedirs(os.path.dirname(self.output))
				
			_output = open(self.output,'w')

		elif self.silent:
			_output = Silencer()
		
		else:
			_output = sys.stdout
			 
		_output.write(result)

		if self.output:
			_output.close()
			
		return result
			

#________________________________________________________________
@args.command(name='assets')
class Assets(STEP):
	'''
	MIME type assets
	'''
	
	base = 'assets'
	
	def __init__(self, asXML=None, verbose=None, output=None, silent=True):
		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent)
		

	@args.operation
	def get(self, id):
		'''
		get the asset by ID
		'''
		return super().get('%s/%s'%(self.base,id))


	@args.operation
	def content(self, id):
		'''
		downlaod the asset to a local directory
		'''
		path='%s/%s/content'%(self.base, id)
		name='/%s/%s/%s/content'%(self.path, self.base, id)
		if not os.path.isdir(os.path.dirname(name)):
			os.makedirs(os.path.dirname(name))
		super().file(path=path,destination=name)


	@args.operation
	def upload(self, id, file):
		pass


	@args.operation
	def update(self, id, file):
		with open(file) as input:
			body = input.read()
			headers = { 'Content-Type' : 'application/octet-stream' }
			return super().put('%s/%s/content'%(self.base, id), body=body, headers=headers)
	

#________________________________________________________________
@args.command(name='processes')
class Processes(STEP):
	'''
	background processes running on STEP
	'''
	base = 'backgroundprocesses'

	def __init__(self, asXML=None, verbose=None, output=None, silent=True):
		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent)


	@args.operation
	def list(self):
		'''
		list the background processes
		'''
		return super().get(self.base)


	@args.operation
	def get(self, id):
		'''
		get the process by id
		'''
		return super().get('%s/%s'%(self.base,id))


#________________________________________________________________
@args.command(name='instances')
class Instances(STEP):
	'''
	background processes instances running on STEP
	'''
	base = 'bgpinstance'

	def __init__(self, asXML=None, verbose=None, output=None, silent=True):
		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent)


	@args.operation
	def list(self):
		'''
		return a list of instances
		'''
		return super().get(self.base)
	

	@args.operation
	def report(self, id):
		'''
		report on the process id
		'''
		return super().get('%s/%s/executionreport'%(self.base,id))


	@args.operation
	def status(self, id):
		'''
		get the states of the process id
		'''
		return super().get('%s/%s/status'%(self.base,id))


	@args.operation
	def attachment(self, id, attchmentID):
		'''
		get the atachment by attachmentID for the process id
		'''
		return super().get('%s/%s/attachment/%s'%(self.base,id,attachmentID))


#________________________________________________________________
@args.command(name='objects')
class ObjectsByKey(STEP):

	base = 'objectsbykey'
	
	def __init__(self, asXML=None, verbose=None, output=None, silent=True):
		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent)


	@args.operation(help='get object by keyid and key')
	@args.parameter(name='keyid', help='the ID of object type')
	@args.parameter(name='key', help='the key of the object')
	def get(self, keyid, key):
		return super().get('%s/%s/%s'%(self.base,keyid,key))


#________________________________________________________________
@args.command(name='products')
class Products(STEP):

	base = 'products'

	def __init__(self, asXML=None, verbose=None, output=None, silent=True):
		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent)
		

	@args.operation(help='list the products')
	@args.parameter(name='root',short='r')
	def list(self,root='ProductsRoot'):
		return super().get('%s/%s/children'%(self.base,root))


	@args.operation(help='get product by id')
	@args.parameter(name='id', help='the ID of product')
	@args.parameter(name='save', short='s', flag=True, help='save to local /restapi')
	def get(self, id, save=False):
		result = super().get('%s/%s'%(self.base, id))
		if save:
			with open('/restapi/products/%s'%id,'w') as output:
				output.write(result)
		return result

	
	@args.operation(help='create a new product')
	@args.parameter(name='name', short='n')
	@args.parameter(name='values', short='a', nargs='*', metavar='attr=value')
	def create(self, parent, objectType, name=None, values=[]):
		body=dict(
			parent=parent,
			objectType=objectType,
			name=name,
			values=dict()
		)
		for nvp in values:
			(attr,value) = tuple(nvp.split('='))
			body['values'][attr] = dict(value=dict(value=value))
		if self.verbose:
			json.dump(body, sys.stderr, indent=4)
		result = super().post('%s'%self.base, body=json.dumps(body))
		result = json.loads(result.text)
		return result

			
	@args.operation(help='delete product by id')
	@args.parameter(name='id', help='the ID of product')
	def delete(self, id):
		result = super().delete('%s/%s'%(self.base, id))
		return result


	@args.operation(help='get children of product by id')
	@args.parameter(name='id', help='the ID of product')
	def children(self, id):
		return super().get('%s/%s/children'%(self.base,id))


	@args.operation(help='get values of product by id')
	@args.parameter(name='id', help='the ID of product')
	@args.parameter(name='attributeID', help='the ID of product')
	def values(self, id, attributeID):
		return super().get('%s/%s/values/%s'%(self.base, id, attributeID))


	@args.operation(help='set values of product by id')
	@args.parameter(name='id', help='the ID of product')
	@args.parameter(name='attributeID', help='the ID of product')
	@args.parameter(name='value', help='the ID of product')
	def update(self, id, attributeID, value):
		headers={
			"accept": "application/json",
			"Content-Type": "application/json",
		}
		payload={
			"value": {
				"value":value
			}
		}
		return super().put('%s/%s/values/%s'%(
			self.base, id, attributeID), body=json.dumps(payload), headers=headers
		)

	@args.operation(help='get tables of product by id')
	@args.parameter(name='id', help='the ID of product')
	def tables(self, id):
		return super().get('%s/%s/tables'%(self.base,id))


	@args.operation(help='search for a product')
	@args.parameter(name='expression', help='the search expression * allowed')
	def search(self, expression):
		return super().get('basicsearch/%s'%expression)


#________________________________________________________________
@args.command(name='entities')
class Entities(STEP):

	base = 'entities'

	def __init__(self, asXML=None, verbose=None, output=None, silent=True):
		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent)

	@args.property(short='r', default='Entity hierarchy root')
	def root(self): return
	
	@args.operation(help='list of children of entity hierarchy root')
	def list(self):
		return super().get('%s/%s/children'%(self.base,self.root))


	@args.operation(help='get entity by id')
	@args.parameter(name='id', help='the ID of entity')
	def get(self, id):
		return super().get('%s/%s'%(self.base,id))


	@args.operation(help='get children of entity by id')
	@args.parameter(name='id', help='the ID of entity')
	def children(self, id):
		return super().get('%s/%s/children'%(self.base,id))


	@args.operation(help='get values of entity by id')
	@args.parameter(name='id', help='the ID of entity')
	def values(self, id):
		return super().get('%s/%s/values'%(self.base,id))


	@args.operation(help='update values of entity by id')
	@args.parameter(name='id', help='the ID of entity')
	@args.parameter(name='name', help='attribute ID')
	@args.parameter(name='value', help='attribute Value')
	def update(self, id, name, value):
		body = json.dumps(dict(value=dict(value=value)))
		headers = { 'Content-Type' : 'application/json' }
		return super().put('%s/%s/values/%s'%(self.base, id, name), body=body, headers=headers)
		

#________________________________________________________________
@args.command(name='classifications')
class Classifications(STEP):

	base = 'classifications'

	def __init__(self, asXML=None, verbose=None, output=None, silent=True):
		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent)

	@args.property(short='r', default='Metcash_Root_Metcash')
	def root(self): return
	
	@args.operation(help='list of children of classification hierarchy root')
	def list(self):
		return super().get('%s/%s/children'%(self.base,self.root))


	@args.operation(help='get classification by id')
	@args.parameter(name='id', help='the ID of classification')
	def get(self, id):
		return super().get('%s/%s'%(self.base,id))


	@args.operation(help='get children of classification by id')
	@args.parameter(name='id', help='the ID of classification')
	def references(self, parent_id, reference_id):
		return super().get('%s/%s/incoming-references/%s'%(self.base, parent_id, reference_id))

	
	@args.operation(help='get children of classification by id')
	@args.parameter(name='id', help='the ID of classification')
	def children(self, id):
		return super().get('%s/%s/children'%(self.base,id))

	
	@args.operation(help='get values of classification by id')
	@args.parameter(name='id', help='the ID of classification')
	def values(self, id):
		return super().get('%s/%s/values'%(self.base,id))
	

#________________________________________________________________
@args.command(name='endpoints')
class Endpoints(STEP):

	base = 'integrationendpoints'

	def __init__(self, asXML=None, verbose=None, output=None, silent=True):
		super().__init__(asXML=asXML, verbose=verbose, output=output, silent=silent)


	@args.operation(help='get a list of endpoints')
	def list(self):
		return super().get('%s'%self.base)


	@args.operation(help='get the logs')
	def log(self, id):
		return super().get('%s/%s/log'%(self.base, id))
		

	@args.operation(help='get errors')
	def errors(self, id):
		return super().get('%s/%s/errorexcerpts'%(self.base, id))
		

	@args.operation(help='get background processes')
	def processes(self, id):
		return super().get('%s/%s/backgroundprocesses'%(self.base, id))
	

	@args.operation(help='invoke an endpoint')
	def invoke(self, id):
		return super().put('%s/%s/invoke'%(self.base, id))
											   




#!/usr/bin/env python3

import os, codecs
from os import path
from setuptools import setup

pwd = path.abspath(path.dirname(__file__))
with codecs.open(path.join(pwd, 'README.md'), 'r', encoding='utf8') as input:
	long_description = input.read()

name='STEP.py'
user='eddo888'
version='1.12'

setup(
	name=name,
	version=version,
	license='MIT',
	long_description=long_description,
	long_description_content_type="text/markdown",
	url='https://github.com/%s/%s'%(user,name),
	download_url='https://github.com/%s/%s/archive/%s.tar.gz'%(user, name, version),
	author='David Edson',
	author_email='eddo888@tpg.com.au',
	packages=[
		'STEP',
	],
	install_requires=[
		'argcomplete',
		'xlrd',
		'xlwt',
		'dotmap',
		'pyxb',
		'suds-py3',
		'xmltodict',
		'Baubles',
		'Perdy',
		'Spanners',
		'Argumental',
		'GoldenChild',
		'Swapsies',
	],
	scripts=[
		'bin/step.soap.py',
		'bin/step.rest.py',
		'bin/excel2step.py',
		'bin/step2uml.py',
	], 
)

